// Diagonal Streams
//
// By Brian Prentice
// April 2003
//
// The rule has the following states:
//  0 background
//  1 stream moving northwest
//  2 stream moving northeast
//  3 stream moving southeast
//  4 stream moving southwest
//  5 .. 6 gate posts
//  7 .. 22 gate selectors

library DiagonalStreams;

uses
  SysUtils,
  Classes,
  Controls,
  Dialogs;

const
  MaxState = 22;

function CARule(Generation,col,row,
                NW,N, NE,
                W, Me,E,
                SW,S, SE: Integer): Integer; stdcall;

var
  R: Integer;

function Gate(var R: Integer): Boolean;

function Lookup(A, B, I: Integer): Integer;

const
  T: Array[0..15,0..3] of Integer = ((0, 0, 0, 0),
                                     (0, 0, 0, 1),
                                     (0, 0, 1, 0),
                                     (0, 0, 1, 1),
                                     (0, 1, 0, 0),
                                     (0, 1, 0, 1),
                                     (0, 1, 1, 0),
                                     (0, 1, 1, 1),
                                     (1, 0, 0, 0),
                                     (1, 0, 0, 1),
                                     (1, 0, 1, 0),
                                     (1, 0, 1, 1),
                                     (1, 1, 0, 0),
                                     (1, 1, 0, 1),
                                     (1, 1, 1, 0),
                                     (1, 1, 1, 1));

var
  J: Integer;

begin
  if A = 0 then
    J := 0
  else
    J := 2;
  if B <> 0 then
    Inc(J);
  Result := T[I, J]
end;

begin
    Result := False;
    if (SW = 5) and (S >= 7) and (S <= 22) then
    begin
      R := Lookup(NW, W, S - 7) * 3;
      Result := True
    end
    else if (NW = 6) and (N >= 7) and (N <= 22) then
    begin
      R := Lookup(W, SW, N - 7) * 2;
      Result := True
    end
    else if (NW = 5) and (W >= 7) and (W <= 22) then
    begin
      R := Lookup(NE, N, W - 7) * 4;
      Result := True
    end
    else if (NE = 6) and (E >= 7) and (E <= 22) then
    begin
      R := Lookup(N, NW, E - 7) * 3;
      Result := True
    end
    else if (NE = 5) and (N >= 7) and (N <= 22) then
    begin
      R := Lookup(SE, E, N - 7);
      Result := True
    end
    else if (SE = 6) and (S >= 7) and (S <= 22) then
    begin
      R := Lookup(E, NE, S - 7) * 4;
      Result := True
    end
    else if (SE = 5) and (E >= 7) and (E <= 22) then
    begin
      R := Lookup(SW, S, E - 7) * 2;
      Result := True
    end
    else if (SW = 6) and (W >= 7) and (W <= 22) then
    begin
      R := Lookup(S, SE, W - 7);
      Result := True
    end
end;

begin
  Result := 0;
  if Gate(R) then
  begin
    Result := R;
    Exit
  end;    
  if Me = 0 then
    if (SE = 1) and (NE <> 4) then
      Result := 1
    else if SW = 2 then
      Result := 2
    else if NW = 3 then
      Result := 3
    else if NE = 4 then
      Result := 4;
  if (Me = 1) and (SE = 1) then
    Result := 1;
  if (Me = 2) and (SW = 2) then
    Result := 2;
  if (Me = 3) and (NW = 3) then
    Result := 3;
  if (Me = 4) and (NE = 4) then
    Result := 4;
  if (Me >= 5) and (Me <= 22) then
    Result := Me    
end;

procedure CASetup(var RuleType, CountOfColors: Integer; ColorPalette,
                  Misc: PChar); stdcall;
begin
  RuleType := 2;
  CountOfColors := MaxState + 1;
  StrCopy(ColorPalette, '')
end;

exports
  CARule   index 1,
  CASetup  index 2;

begin
end.

