// Particle Streams
// Version 2
//
// By Brian Prentice
// March 2003
//
// The rule has the following states:
//  0 background
//  1 stream moving north
//  2 stream moving east
//  3 stream moving south
//  4 stream moving west
//  5 stream terminator
//  6 to 21 gates

library ParticleStreams2;

uses
  SysUtils,
  Classes,
  Controls,
  Dialogs;

const
  MaxState = 11;
  G_AND = 1;
  G_OR = 7;
  G_XOR = 6;
  G_NAND = 14;
  G_NOR = 8;
  G_XNOR = 9;

function CARule(Generation,col,row,
                NW,N, NE,
                W, Me,E,
                SW,S, SE: Integer): Integer; stdcall;

var
  R: Integer;

function Gate(State, I: Integer; var R: Integer): Boolean;

function Lookup(A, B: Integer): Integer;

const
  T: Array[0..15,0..3] of Integer = ((0, 0, 0, 0),
                                     (0, 0, 0, 1),
                                     (0, 0, 1, 0),
                                     (0, 0, 1, 1),
                                     (0, 1, 0, 0),
                                     (0, 1, 0, 1),
                                     (0, 1, 1, 0),
                                     (0, 1, 1, 1),
                                     (1, 0, 0, 0),
                                     (1, 0, 0, 1),
                                     (1, 0, 1, 0),
                                     (1, 0, 1, 1),
                                     (1, 1, 0, 0),
                                     (1, 1, 0, 1),
                                     (1, 1, 1, 0),
                                     (1, 1, 1, 1));

var
  J: Integer;

begin
  if A = 0 then
    J := 0
  else
    J := 2;
  if B <> 0 then
    Inc(J);
  Result := T[I, J]
end;

begin
    Result := False;
    if (S = State) and
       ((SW = 2) or (SW = 0)) and ((SE = 4) or (SE = 0)) then
    begin
      R := Lookup(SW, SE);
      Result := True
    end
    else if (W = State) and
            ((NW = 3) or (NW = 0)) and ((SW = 1) or (SW = 0)) then
    begin
      R := Lookup(NW, SW) * 2;
      Result := True
    end
    else if (N = State) and
            ((NE = 4) or (NE = 0)) and ((NW = 2) or (NW = 0)) then
    begin
      R := Lookup(NE, NW) * 3;
      Result := True
    end
    else if (E = State) and
            ((SE = 1) or (SE = 0)) and ((NE = 3) or (NE = 0)) then
    begin
      R := Lookup(SE, NE) * 4;
      Result := True
    end
end;

begin
  Result := 0;
  if ((Me = 0) or (Me = 1)) and (S = 1) and (E <> 4) then
    Result := 1
  else if ((Me = 0) or (Me = 2)) and (W = 2) then
    Result := 2
  else if ((Me = 0) or (Me = 3)) and (N = 3) then
    Result := 3
  else if ((Me = 0) or (Me = 4)) and (E = 4) then
    Result := 4
  else if Gate(6, G_XOR, R) then
    Result := R
  else if Gate(7, G_OR, R) then
    Result := R
  else if Gate(8, G_AND, R) then
    Result := R
  else if Gate(9, G_NAND, R) then
    Result := R
  else if Gate(10, G_NOR, R) then
    Result := R
  else if Gate(11, G_XNOR, R) then
    Result := R;
  if ((Me = 1) and (S = 0)) or ((Me = 2) and (W = 0)) or
     ((Me = 3) and (N = 0)) or ((Me = 4) and (E = 0)) then
    Result := 0;
  if (Me >= 5) then
    Result := Me
end;

procedure CASetup(var RuleType, CountOfColors: Integer; ColorPalette,
                  Misc: PChar); stdcall;
begin
  RuleType := 2;
  CountOfColors := MaxState + 1;
  StrCopy(ColorPalette, '')
end;

exports
  CARule   index 1,
  CASetup  index 2;

begin
end.

